unit PrintDialogUnit;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Dialogs, Buttons, ExtCtrls, wNotePad, NotePadPrinter, Spin;

type
  TPrintDialog = class(TForm)
    CancelBtn: TButton;
    Button_Print: TButton;
    Button_Setup: TButton;
    Label_PrinterInfo: TLabel;
    PrinterSetupDialog1: TPrinterSetupDialog;
    Button_Preview: TButton;
    Label10: TLabel;
    Label11: TLabel;
    ComboBox_Font: TComboBox;
    ComboBox_FontSize: TComboBox;
    CheckBox_LineNumber: TCheckBox;
    ComboBox_PrintMode: TComboBox;
    GroupBox3: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label24: TLabel;
    SpinEdit_MBottom: TSpinEdit;
    SpinEdit_MTop: TSpinEdit;
    SpinEdit_MRight: TSpinEdit;
    SpinEdit_MLeft: TSpinEdit;
    SpinEdit_MLine: TSpinEdit;
    GroupBox2: TGroupBox;
    Label_StartPage: TLabel;
    Label_LastPage: TLabel;
    SpinEdit_StartPage: TSpinEdit;
    SpinEdit_LastPage: TSpinEdit;
    CheckBox_PageLimit: TCheckBox;
    GroupBox1: TGroupBox;
    Label7: TLabel;
    RadioGroup_ColumnCount: TRadioGroup;
    SpinEdit_ColumnSpace: TSpinEdit;
    GroupBox4: TGroupBox;
    Label8: TLabel;
    Label9: TLabel;
    RadioGroup_Header: TRadioGroup;
    RadioGroup_Footer: TRadioGroup;
    Edit_Header: TComboBox;
    Edit_Footer: TComboBox;
    Button3: TButton;
    Label5: TLabel;
    Label6: TLabel;
    Label12: TLabel;
    Label13: TLabel;
    Label14: TLabel;
    Label15: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button_PrintClick(Sender: TObject);
    procedure Button_SetupClick(Sender: TObject);
    procedure Button_PreviewClick(Sender: TObject);
    procedure ComboBox_FontDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure ComboBox_FontChange(Sender: TObject);
    procedure ComboBox_FontSizeChange(Sender: TObject);
    procedure ComboBox_PrintModeChange(Sender: TObject);
    procedure CheckBox_LineNumberClick(Sender: TObject);
    procedure RadioGroup_ColumnCountClick(Sender: TObject);
    procedure SpinEdit_ColumnSpaceChange(Sender: TObject);
    procedure SpinEdit_MLeftChange(Sender: TObject);
    procedure SpinEdit_MRightChange(Sender: TObject);
    procedure SpinEdit_MTopChange(Sender: TObject);
    procedure SpinEdit_MBottomChange(Sender: TObject);
    procedure SpinEdit_MLineChange(Sender: TObject);
    procedure RadioGroup_HeaderClick(Sender: TObject);
    procedure RadioGroup_FooterClick(Sender: TObject);
    procedure Edit_HeaderChange(Sender: TObject);
    procedure Edit_FooterChange(Sender: TObject);
    procedure CheckBox_PageLimitClick(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure SpinEdit_StartPageChange(Sender: TObject);
    procedure SpinEdit_LastPageChange(Sender: TObject);
  private
    FNotePadPrint: TNotePadPrinter;
    procedure SetNotePad(const Value: TwNotePad);
    procedure InfoLabelUpdate;
    function GetFileName: WideString;
    procedure SetFileName(const Value: WideString);
    { Private 錾 }
  public
    procedure ReadOptions;
    procedure WriteOptions;
    property NotePad: TwNotePad write SetNotePad;
    property FileName: WideString read GetFileName write SetFileName;
    { Public 錾 }
  end;

var
  FPrinterFontName: WideString;
  FPrinterFontSize: integer;
  FPrinterLineNumber: boolean;
  FPrinterPrintMode: integer;
  FPrinterColumnCount: integer;
  FPrinterColumnSpace: integer;
  FPrinterMLeft: integer;
  FPrinterMTop: integer;
  FPrinterMRight: integer;
  FPrinterMBottom: integer;
  FPrinterMLine: integer;
  FPrinterHeader: WideString;
  FPrinterHeaderPos: integer;
  FPrinterFooter: WideString;
  FPrinterFooterPos: integer;

implementation

uses types, Printers, Math, FontUtils;

{$R *.dfm}

procedure TPrintDialog.FormCreate(Sender: TObject);
var
  i: integer;
  h: HDC;
begin
  FNotePadPrint := TNotePadPrinter.Create(self);
  InfoLabelUpdate;
  // font name
  h := GetDC(0);
  ComboBox_Font.Items.BeginUpdate;
  try
    ComboBox_Font.Items.Clear;
    try
      for i := 0 to Printer.Fonts.Count - 1 do
        if Printer.Fonts[i][1] <> '@' then begin
          if IsFixedFontName(Printer.Fonts[i], h) then
            ComboBox_Font.Items.Add(Printer.Fonts[i]);
      end;
    except
      // ݂̃v^[ł́AScreen.Fonts 񋓂
      for i := 0 to Screen.Fonts.Count - 1 do
        if Screen.Fonts[i][1] <> '@' then begin
          if IsFixedFontName(Screen.Fonts[i], h) then
            ComboBox_Font.Items.Assign(Screen.Fonts);
      end;
    end;

  finally
    ComboBox_Font.Items.EndUpdate;
  end;
  ReleaseDC(0, h);
  ComboBox_Font.Sorted := true;
  ComboBox_Font.ItemIndex := ComboBox_Font.Items.IndexOf(FNotePadPrint.Font.Name);
  // font size
  for I := 6 to 20 do
    ComboBox_FontSize.Items.Add(IntToStr(i));
  ComboBox_FontSize.ItemIndex := ComboBox_FontSize.Items.IndexOf(IntToStr(FNotePadPrint.Font.Size));
  ComboBox_PrintMode.ItemIndex := Integer(FNotePadPrint.PrintMode);
  CheckBox_LineNumber.Checked := FNotePadPrint.LineNumber;
  RadioGroup_ColumnCount.ItemIndex := FNotePadPrint.ColumnCount - 1;
  SpinEdit_ColumnSpace.Value := FNotePadPrint.ColumnSpace;
  SpinEdit_MLeft.Value := FNotePadPrint.MarginLeft;
  SpinEdit_MRight.Value := FNotePadPrint.MarginRight;
  SpinEdit_MTop.Value := FNotePadPrint.MarginTop;
  SpinEdit_MBottom.Value := FNotePadPrint.MarginBottom;
  SpinEdit_MLine.Value := FNotePadPrint.MarginLine;
  RadioGroup_Header.ItemIndex := integer(FNotePadPrint.HeaderAlign);
  RadioGroup_Footer.ItemIndex := integer(FNotePadPrint.FooterAlign);
  Edit_Header.Text := FNotePadPrint.Header;
  Edit_Footer.Text := FNotePadPrint.Footer;
  CheckBox_PageLimit.Checked := FNotePadPrint.RangeOption = roLimited;
  SpinEdit_StartPage.Value := FNotePadPrint.StartPage;
  SpinEdit_LastPage.Value := FNotePadPrint.LastPage;
end;

procedure TPrintDialog.FormDestroy(Sender: TObject);
begin
  FNotePadPrint.Free;
end;

procedure TPrintDialog.SetNotePad(const Value: TwNotePad);
begin
  FNotePadPrint.NotePad := Value;
end;

procedure TPrintDialog.Button_PrintClick(Sender: TObject);
begin
  FNotePadPrint.PrintExecute;
end;

procedure TPrintDialog.Button_SetupClick(Sender: TObject);
begin
  PrinterSetupDialog1.Execute;
  InfoLabelUpdate;
end;

procedure TPrintDialog.InfoLabelUpdate;
begin
  Label_PrinterInfo.Caption := FNotePadPrint.PrinterInfo.PrinterName+#13#10+
    FNotePadPrint.PrinterInfo.PaperName;
  try
    if Printer.Orientation = poPortrait then
      Label_PrinterInfo.Caption := Label_PrinterInfo.Caption+ ' c'
    else
      Label_PrinterInfo.Caption := Label_PrinterInfo.Caption+ ' ';
  except
    // ݂̃v^[ł́ApoPortrait ɌŒ
    Label_PrinterInfo.Caption := Label_PrinterInfo.Caption+ ' c';
  end;

end;

procedure TPrintDialog.Button_PreviewClick(Sender: TObject);
begin
  FNotePadPrint.PreviewExecute;
end;

procedure TPrintDialog.ComboBox_FontDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  ws: WideString;
begin
//
  with (Control as TComboBox).Canvas do
  begin
    FillRect(Rect);
    ws := ComboBox_Font.Items[Index];
    ExtTextOutW(Handle, Rect.Left + 25, Rect.Top, ETO_CLIPPED, @Rect, PWChar(ws), Length(ws), nil);
    Font.Name := ws;
    Font.Height := -12;//BDF UM+΍
    Font.Charset := GetFontCharSet(ws);
    case Font.Charset of
      SHIFTJIS_CHARSET:begin
        //TextOutW(Handle, Rect.Left, Rect.Top, '', 1);
        ExtTextOutW(Handle, Rect.Left, Rect.Top, ETO_CLIPPED, @Rect, '', 1, nil);
        //DrawTextW(Handle, '', 1, Rect, 0);
      end;
      else begin
        //TextOutW(Handle, Rect.Left, Rect.Top, 'Aa', 2);
        ExtTextOutW(Handle, Rect.Left, Rect.Top, ETO_CLIPPED, @Rect, 'Aa', 2, nil);
        //DrawTextW(Handle, 'Aa', 2, Rect, 0);
      end;
    end;
    //TextOut(Rect.Left + 5, Rect.Top + 1, ws);
    //ExtTextOutW(Handle, Rect.Left + 5, Rect.Top, ETO_CLIPPED, @Rect,PWChar(ws), Length(ws), nil);
  end;
end;

procedure TPrintDialog.ComboBox_FontChange(Sender: TObject);
begin
  FNotePadPrint.Font.Name := ComboBox_Font.Items[ComboBox_Font.ItemIndex];
end;

procedure TPrintDialog.ComboBox_FontSizeChange(Sender: TObject);
begin
  FNotePadPrint.Font.Size := StrToIntDef(ComboBox_FontSize.Items[ComboBox_FontSize.ItemIndex], 9);
end;

procedure TPrintDialog.ComboBox_PrintModeChange(Sender: TObject);
begin
  FNotePadPrint.PrintMode := TStringsPrinterPrintMode(ComboBox_PrintMode.ItemIndex);
end;

function TPrintDialog.GetFileName: WideString;
begin
  result := FNotePadPrint.FileName;
end;

procedure TPrintDialog.SetFileName(const Value: WideString);
begin
  FNotePadPrint.FileName := Value;
end;

procedure TPrintDialog.CheckBox_LineNumberClick(Sender: TObject);
begin
  FNotePadPrint.LineNumber := CheckBox_LineNumber.Checked;
end;

procedure TPrintDialog.RadioGroup_ColumnCountClick(Sender: TObject);
begin
  FNotePadPrint.ColumnCount := RadioGroup_ColumnCount.ItemIndex + 1;
end;

procedure TPrintDialog.SpinEdit_ColumnSpaceChange(Sender: TObject);
begin
  FNotePadPrint.ColumnSpace := SpinEdit_ColumnSpace.Value;
end;

procedure TPrintDialog.SpinEdit_MLeftChange(Sender: TObject);
begin
  FNotePadPrint.MarginLeft := SpinEdit_MLeft.Value;
end;

procedure TPrintDialog.SpinEdit_MRightChange(Sender: TObject);
begin
  FNotePadPrint.MarginRight := SpinEdit_MRight.Value;
end;

procedure TPrintDialog.SpinEdit_MTopChange(Sender: TObject);
begin
  FNotePadPrint.MarginTop := SpinEdit_MTop.Value;
end;

procedure TPrintDialog.SpinEdit_MBottomChange(Sender: TObject);
begin
  FNotePadPrint.MarginBottom := SpinEdit_MBottom.Value;
end;

procedure TPrintDialog.SpinEdit_MLineChange(Sender: TObject);
begin
  FNotePadPrint.MarginLine := SpinEdit_MLine.Value;
end;

procedure TPrintDialog.RadioGroup_HeaderClick(Sender: TObject);
begin
  FNotePadPrint.HeaderAlign := TStringsPrinterHeaderAlign(RadioGroup_Header.ItemIndex);
end;

procedure TPrintDialog.RadioGroup_FooterClick(Sender: TObject);
begin
  FNotePadPrint.FooterAlign := TStringsPrinterHeaderAlign(RadioGroup_Footer.ItemIndex);
end;

procedure TPrintDialog.Edit_HeaderChange(Sender: TObject);
begin
  FNotePadPrint.Header := Edit_Header.Text;
end;

procedure TPrintDialog.Edit_FooterChange(Sender: TObject);
begin
  FNotePadPrint.Footer := Edit_Footer.Text;
end;

procedure TPrintDialog.CheckBox_PageLimitClick(Sender: TObject);
var
  b: boolean;
begin
  b := CheckBox_PageLimit.Checked;
  if b then begin
    FNotePadPrint.RangeOption := roLimited;
  end else begin
    FNotePadPrint.RangeOption := roWhole;
  end;
  Label_StartPage.Enabled := b;
  Label_LastPage.Enabled := b;
  SpinEdit_StartPage.Enabled := b;
  SpinEdit_LastPage.Enabled := b;
end;

procedure TPrintDialog.Button3Click(Sender: TObject);
begin
  ShowMessage(
    'wb_[/tb^[ɗpoL[[h͈ȉ̒ʂłB' + #13#10#13#10 +
    '%p  y[Wԍ' + #13#10 +
    '%f  t@CpXt' + #13#10 +
    '%F  t@CpX' + #13#10 +
    '%y  N' + #13#10 +
    '%m  ' + #13#10 +
    '%d  ' + #13#10 +
    '%t  '
  );
end;

procedure TPrintDialog.SpinEdit_StartPageChange(Sender: TObject);
begin
  FNotePadPrint.StartPage := SpinEdit_StartPage.Value;
end;

procedure TPrintDialog.SpinEdit_LastPageChange(Sender: TObject);
begin
  FNotePadPrint.LastPage := SpinEdit_LastPage.Value;
end;

procedure TPrintDialog.ReadOptions;
begin
  ComboBox_Font.ItemIndex := ComboBox_Font.Items.IndexOf(FPrinterFontName);
  FNotePadPrint.Font.Name := FPrinterFontName;
  ComboBox_FontSize.ItemIndex := ComboBox_FontSize.Items.IndexOf(IntToStr(FPrinterFontSize));
  FNotePadPrint.Font.Size := FPrinterFontSize;
  ComboBox_PrintMode.ItemIndex := FPrinterPrintMode;
  FNotePadPrint.PrintMode := TStringsPrinterPrintMode(FPrinterPrintMode);
  CheckBox_LineNumber.Checked := FPrinterLineNumber;
  FNotePadPrint.LineNumber := FPrinterLineNumber;
  RadioGroup_ColumnCount.ItemIndex := Max(1, FPrinterColumnCount) - 1;
  FNotePadPrint.ColumnCount := RadioGroup_ColumnCount.ItemIndex + 1;
  SpinEdit_ColumnSpace.Value := FPrinterColumnSpace;
  FNotePadPrint.ColumnSpace := FPrinterColumnSpace;
  SpinEdit_MLeft.Value := FPrinterMLeft;
  FNotePadPrint.MarginLeft := FPrinterMLeft;
  SpinEdit_MRight.Value := FPrinterMRight;
  FNotePadPrint.MarginRight := FPrinterMRight;
  SpinEdit_MTop.Value := FPrinterMTop;
  FNotePadPrint.MarginTop := FPrinterMTop;
  SpinEdit_MBottom.Value := FPrinterMBottom;
  FNotePadPrint.MarginBottom := FPrinterMBottom;
  SpinEdit_MLine.Value := FPrinterMLine;
  FNotePadPrint.MarginLine := FPrinterMLine;
  RadioGroup_Header.ItemIndex := FPrinterHeaderPos;
  FNotePadPrint.HeaderAlign := TStringsPrinterHeaderAlign(FPrinterHeaderPos);
  RadioGroup_Footer.ItemIndex := FPrinterFooterPos;
  FNotePadPrint.FooterAlign := TStringsPrinterHeaderAlign(FPrinterFooterPos);
  Edit_Header.Text := FPrinterHeader;
  FNotePadPrint.Header := FPrinterHeader;
  Edit_Footer.Text := FPrinterFooter;
  FNotePadPrint.Footer := FPrinterFooter;
end;

procedure TPrintDialog.WriteOptions;
begin
  FPrinterFontName := ComboBox_Font.Items[ComboBox_Font.ItemIndex];
  FPrinterFontSize := StrToIntDef(ComboBox_FontSize.Items[ComboBox_FontSize.ItemIndex], 9);
  FPrinterPrintMode := ComboBox_PrintMode.ItemIndex;
  FPrinterLineNumber := CheckBox_LineNumber.Checked;
  FPrinterColumnCount := RadioGroup_ColumnCount.ItemIndex + 1;
  FPrinterColumnSpace := SpinEdit_ColumnSpace.Value;
  FPrinterMLeft := SpinEdit_MLeft.Value;
  FPrinterMTop := SpinEdit_MTop.Value;
  FPrinterMRight := SpinEdit_MRight.Value;
  FPrinterMBottom := SpinEdit_MBottom.Value;
  FPrinterMLine := SpinEdit_MLine.Value;
  FPrinterHeader := Edit_Header.Text;
  FPrinterFooter := Edit_Footer.Text;
  FPrinterHeaderPos := RadioGroup_Header.ItemIndex;
  FPrinterFooterPos := RadioGroup_Footer.ItemIndex;
end;

end.
