unit ModeBat;

interface

uses Windows, NotePadMode, NotePadString;

type
  TModeBat = class(TNotePadMode)
  protected
    procedure InitReserveList;override;
  public
    function NewParser(target: TNotePadString): TNotePadParser;override;
  end;

  TParserBat = class(TNotePadParser)
  private
  protected
    procedure ParseToken(AHttp: boolean);overload;override;
    procedure ParseToken(const C: WideChar);overload;override;
  public
    constructor Create(target: TNotePadString; mode: TModeBat);
  end;

implementation

uses NotePadClasses, ParserConsts;

const
  ss1: array[0..12] of WideString =(
'call',
'do',
'echo',
'else',
'endlocal',
'for',
'goto',
'if',
'in',
'not',
'pause',
'setlocal',
'shift');

  ss2: array[0..60] of WideString =(
'assoc',
'at',
'attrib',
'break',
'cacls',
'cd',
'chcp',
'chdir',
'chkdsk',
'chkntfs',
'cls',
'cmd',
'color',
'comp',
'compact',
'convert',
'copy',
'date',
'del',
'dir',
'diskcomp',
'diskcopy',
'doskey',
'erase',
'exit',
'fc',
'find',
'findstr',
'format',
'ftype',
'graftabl',
'help',
'label',
'md',
'mkdir',
'mode',
'more',
'move',
'path',
'popd',
'print',
'prompt',
'pushd',
'rd',
'recover',
'ren',
'rename',
'replace',
'rmdir',
'set',
'sort',
'start',
'subst',
'time',
'title',
'tree',
'type',
'ver',
'verify',
'vol',
'xcopy');

type
  THash1 = class(THashTable)
  protected
    procedure InitTable; override;
  public
    function isReserbed(const Value: WideString): boolean; override;
  end;

{ THash1 }

procedure THash1.InitTable;
var
  aa1: array of word;
  i, j: integer;
begin
  FSize := 29;{}
  FMaxLen := 1;
  SetLength(aa1, Length(ss1{}));
  for i := 0 to  Length(aa1) - 1 do begin
    aa1[i] := AnkToHash(ss1{}[i]);
    if Length(ss1{}[i]) > FMaxLen then FMaxLen := Length(ss1{}[i]);
  end;
  SetLength(FTable, FSize);
  for i := 0 to  Length(FTable) - 1 do begin
    FTable[i] := 0;
  end;
  for i := 0 to  Length(aa1) - 1 do begin
    j := aa1[i];
    while FTable[j] > 0 do begin
      inc(j);
      if j >= Length(FTable) then dec(j, Length(FTable) - 1);
    end;
    FTable[j] := i + 1;
  end;
end;

function THash1.isReserbed(const Value: WideString): boolean;
var
  i: integer;
begin
  Result := false;
  if Length(Value) > FMaxLen then exit;
  i := AnkToHash(Value);
  while FTable[i] > 0 do begin
    if CharLowerW(PWChar(ss1{}[FTable[i] - 1])) = Value then begin
      Result := true;
      exit;
    end;
    inc(i);
    if i >= FSize then dec(i, FSize);
  end;
end;

type
  THash2 = class(THashTable)
  protected
    procedure InitTable; override;
  public
    function isReserbed(const Value: WideString): boolean; override;
  end;

{ THash2 }

procedure THash2.InitTable;
var
  aa1: array of word;
  i, j: integer;
begin
  FSize := 127;{}
  FMaxLen := 1;
  SetLength(aa1, Length(ss2{}));
  for i := 0 to  Length(aa1) - 1 do begin
    aa1[i] := AnkToHash(ss2{}[i]);
    if Length(ss2{}[i]) > FMaxLen then FMaxLen := Length(ss2{}[i]);
  end;
  SetLength(FTable, FSize);
  for i := 0 to  Length(FTable) - 1 do begin
    FTable[i] := 0;
  end;
  for i := 0 to  Length(aa1) - 1 do begin
    j := aa1[i];
    while FTable[j] > 0 do begin
      inc(j);
      if j >= Length(FTable) then dec(j, Length(FTable) - 1);
    end;
    FTable[j] := i + 1;
  end;
end;

function THash2.isReserbed(const Value: WideString): boolean;
var
  i: integer;
begin
  Result := false;
  if Length(Value) > FMaxLen then exit;
  i := AnkToHash(Value);
  while FTable[i] > 0 do begin
    if CharLowerW(PWChar(ss2{}[FTable[i] - 1])) = Value then begin
      Result := true;
      exit;
    end;
    inc(i);
    if i >= FSize then dec(i, FSize);
  end;
end;

var
  FHash1: THash1;
  FHash2: THash2;

type
  TBatList = class(TReserveList)
  public
    function Find(const s: WideString): boolean; override;
  end;

{ TBatList }

function TBatList.Find(const s: WideString): boolean;
begin
  Result := FHash1.isReserbed(CharLowerW(PWChar(s)));
end;

type
  TBat2List = class(TReserveList)
  public
    function Find(const s: WideString): boolean; override;
  end;

{ TBat2List }

function TBat2List.Find(const s: WideString): boolean;
begin
  Result := FHash2.isReserbed(CharLowerW(PWChar(s)));
end;

{ TModeBat }

procedure TModeBat.InitReserveList;
var
  a: array of WideString;
begin
  FReserveList := TBatList.Create(a, false);
  FReserve2List := TBat2List.Create(a, true);
  FReserve3List := TReserveList.Create(a, true);
end;

function TModeBat.NewParser(target: TNotePadString): TNotePadParser;
begin
  Result := TParserBat.Create(target, self);
end;

{ TParserBat }

constructor TParserBat.Create(target: TNotePadString; mode: TModeBat);
begin
  inherited Create(target, mode);
end;

procedure TParserBat.ParseToken(AHttp: boolean);
var
  C2, C3, C4: WideChar;
begin
  case FSb[FParsePos] of
    '"': begin
      if FParsePos < FSb.StringLength - 1 then begin
        C2 := FSb[FParsePos + 1];
        if C2 <> #10 then begin
          inc(FParsePos);
          ParseToken(toDoubleQuotation);
          exit;
        end;
      end;
      ParseToken(toDoubleQuotation);
      exit;
    end;
    'R', 'r': begin
      if FParsePos < FSb.StringLength - 2 then begin
        C2 := FSb[FParsePos + 1];
        C3 := FSb[FParsePos + 2];
        if ((C2 = WideChar('E')) or (C2 = WideChar('e')))
           and ((C3 = WideChar('M')) or (C3 = WideChar('m'))) then begin
          if FParsePos < FSb.StringLength - 3 then begin
            C4 := FSb[FParsePos + 3];
            if (C4 >= WideChar('0')) and (C4 <= WideChar('z')) and
              ((C4 >= WideChar('a')) or (C4 = WideChar('_')) or
              ((C4 <= WideChar('Z'))and((C4 >= WideChar('A')) or (C4 <= WideChar('9'))))) then begin
              inherited;
              exit;
            end;
          end;
          inc(FParsePos, 2);
          ParseToken(toComment);
          exit;
        end;
      end;
    end;
  end;//case
  inherited ParseToken(false);
end;

procedure TParserBat.ParseToken(const C: WideChar);
begin
  case C of
    toDoubleQuotation: begin
      FToken := C;
      DoubleQuotationProc;
    end;
    toComment: begin
      FToken := C;
      CommentProc;
    end;
    else inherited;
  end;
end;

initialization
  FHash1 := THash1.Create;
  FHash2 := THash2.Create;
finalization
  FHash1.Free;
  FHash2.Free;
end.
