unit ModeCss;

interface

uses Windows, NotePadMode, NotePadString;

type
  TModeCss = class(TNotePadMode)
  protected
    procedure InitReserveList;override;
  public
    function NewParser(target: TNotePadString): TNotePadParser;override;
  end;

  TParserCss = class(TNotePadParser)
  private
  protected
    procedure ParseToken(AHttp: boolean);overload;override;
    procedure ParseToken(const C: WideChar);overload;override;
  public
    constructor Create(target: TNotePadString; mode: TModeCss);
  end;

implementation

uses NotePadClasses, ParserConsts;

const
  ss1: array[0..154] of WideString =(
'azimuth',
'background',
'background-attachment',
'background-color',
'background-image',
'background-position',
'background-position-xIE',
'background-position-yIE',
'background-repeat',
'behaviorIE',
'border',
'border-bottom',
'border-bottom-color',
'border-bottom-style',
'border-bottom-width',
'border-collapse',
'border-color',
'border-left',
'border-left-color',
'border-left-style',
'border-left-width',
'border-right',
'border-right-color',
'border-right-style',
'border-right-width',
'border-spacing',
'border-style',
'border-top',
'border-top-color',
'border-top-style',
'border-top-width',
'border-width',
'bottom',
'caption-side',
'clear',
'clip',
'color',
'content',
'counter-increment',
'counter-reset',
'cue',
'cue-after',
'cue-before',
'cursor',
'direction',
'display',
'elevation',
'empty-cells',
'filterIE',
'float',
'font',
'font-family',
'font-size',
'font-size-adjust',
'font-stretch',
'font-style',
'font-variant',
'font-weight',
'height',
'ime-modeIMEIE',
'include-sourceHTMLNN',
'layer-background-colorNN',
'layer-background-imageNN',
'layout-gridIE',
'layout-grid-charIE',
'layout-grid-lineIE',
'layout-grid-modeIE',
'layout-grid-typeIE',
'left',
'letter-spacing',
'line-breakIE',
'line-height',
'list-style',
'list-style-image',
'list-style-position',
'list-style-type',
'margin',
'margin-bottom',
'margin-left',
'margin-right',
'margin-top',
'marker-offset',
'marks',
'max-height',
'max-width',
'min-height',
'min-width',
'orphans',
'outline',
'outline-color',
'outline-style',
'outline-width',
'overflow',
'overflow-xIE',
'overflow-yIE',
'padding',
'padding-bottom',
'padding-left',
'padding-right',
'padding-top',
'page',
'page-break-before',
'page-break-after',
'page-break-inside',
'pause',
'pause-after',
'pause-before',
'pitch',
'pitch-range',
'play-during',
'position',
'quotes',
'richness',
'right',
'ruby-alignIE',
'ruby-overhangIE',
'ruby-positionIE',
'scrollbardlight-colorIE',
'scrollbar-arrow-colorIE',
'scrollbar-base-colorIE',
'scrollbar-darkshadow-colorIE',
'scrollbar-face-colorIE',
'scrollbar-highlight-colorIE',
'scrollbar-shadow-colorIE',
'size',
'speak',
'speak-header',
'speak-numeral',
'speak-punctuation',
'speech-rate',
'stress',
'table-layout',
'text-align',
'text-autospaceIE',
'text-decoration',
'text-indent',
'text-justifyIE',
'text-shadow',
'text-transform',
'text-underline-positionIE',
'top',
'unicode-bidiUnicode',
'vertical-align',
'visibility',
'voice-family',
'volume',
'white-space',
'widows',
'width',
'word-breakIE',
'word-spacing',
'word-wrapIE',
'writing-modeIE',
'z-index',
'zoomIE');

type
  THash1 = class(THashTable)
  protected
    procedure InitTable; override;
  public
    function isReserbed(const Value: WideString): boolean; override;
  end;

{ THash1 }

procedure THash1.InitTable;
var
  aa1: array of word;
  i, j: integer;
begin
  FSize := 313;{}
  FMaxLen := 1;
  SetLength(aa1, Length(ss1{}));
  for i := 0 to  Length(aa1) - 1 do begin
    aa1[i] := AnkToHash(ss1{}[i]);
    if Length(ss1{}[i]) > FMaxLen then FMaxLen := Length(ss1{}[i]);
  end;
  SetLength(FTable, FSize);
  for i := 0 to  Length(FTable) - 1 do begin
    FTable[i] := 0;
  end;
  for i := 0 to  Length(aa1) - 1 do begin
    j := aa1[i];
    while FTable[j] > 0 do begin
      inc(j);
      if j >= Length(FTable) then dec(j, Length(FTable) - 1);
    end;
    FTable[j] := i + 1;
  end;
end;

function THash1.isReserbed(const Value: WideString): boolean;
var
  i: integer;
begin
  Result := false;
  if Length(Value) > FMaxLen then exit;
  i := AnkToHash(Value);
  while FTable[i] > 0 do begin
    if CharLowerW(PWChar(ss1{}[FTable[i] - 1])) = Value then begin
      Result := true;
      exit;
    end;
    inc(i);
    if i >= FSize then dec(i, FSize);
  end;
end;

var
  FHash1: THash1;

type
  TCssList = class(TReserveList)
  public
    function Find(const s: WideString): boolean; override;
  end;

{ TCssList }

function TCssList.Find(const s: WideString): boolean;
begin
  Result := FHash1.isReserbed(CharLowerW(PWChar(s)));
end;

{ TModeCss }

procedure TModeCss.InitReserveList;
var
  a: array of WideString;
begin
  FReserveList := TCssList.Create(a, false);
  FReserve2List := TReserveList.Create(a, true);
  FReserve3List := TReserveList.Create(a, true);
end;

function TModeCss.NewParser(target: TNotePadString): TNotePadParser;
begin
  Result := TParserCss.Create(target, self);
end;

{ TParserCss }

constructor TParserCss.Create(target: TNotePadString; mode: TModeCss);
begin
  inherited Create(target, mode);
end;

procedure TParserCss.ParseToken(AHttp: boolean);
var
  C2: WideChar;
begin
  if FToken = toCommentBrock1Start then begin
    ParseToken(toCommentBrock1);
    exit;
  end;
  case FSb[FParsePos] of
    '''': begin
      if FParsePos < FSb.StringLength - 1 then begin
        C2 := FSb[FParsePos + 1];
        if C2 <> #10 then begin
          inc(FParsePos);
          ParseToken(toSingleQuotation);
          exit;
        end;
      end;
      ParseToken(toSingleQuotation);
      exit;
    end;
    '"': begin
      if FParsePos < FSb.StringLength - 1 then begin
        C2 := FSb[FParsePos + 1];
        if C2 <> #10 then begin
          inc(FParsePos);
          ParseToken(toDoubleQuotation);
          exit;
        end;
      end;
      ParseToken(toDoubleQuotation);
      exit;
    end;
    '/': begin
      if FParsePos < FSb.StringLength - 1 then begin
        C2 := FSb[FParsePos + 1];
        if C2 = '*' then begin
          inc(FParsePos);
          ParseToken(toCommentBrock1Start);
          exit;
        end;
      end;
    end;
  end;//case
  inherited ParseToken(false);
end;

procedure TParserCss.ParseToken(const C: WideChar);
begin
  case C of
    toAnk: begin
      FToken := C;
      Ank3Proc;
    end;
    toSingleQuotation: begin
      FToken := C;
      SingleQuotSkipProc(false);
    end;
    toSingleQuotSkip: begin
      FToken := toSingleQuotation;
      SingleQuotSkipProc(true);
    end;
    toDoubleQuotation: begin
      FToken := C;
      DoubleQuotSkipProc(false);
    end;
    toDoubleQuotSkip: begin
      FToken := toDoubleQuotation;
      DoubleQuotSkipProc(true);
    end;
    toCommentBrock1Start: begin
      FToken := C;
      CommentBrock1StartProc;
    end;
    toCommentBrock1: begin
      FToken := C;
      CommentBrock1Proc(0);
    end;
    toCommentBrock1sp: begin
      FToken := toCommentBrock1;
      CommentBrock1Proc(1);
    end;
    else inherited;
  end;
end;

initialization
  FHash1 := THash1.Create;
finalization
  FHash1.Free;
end.
