unit ModeJScript;

interface

uses NotePadMode, NotePadString;

type
  TModeJScript = class(TNotePadMode)
  protected
    procedure InitReserveList;override;
  public
    function NewParser(target: TNotePadString): TNotePadParser;override;
  end;

  TParserJScript = class(TNotePadParser)
  private
  protected
    procedure ParseToken(AHttp: boolean);overload;override;
    procedure ParseToken(const C: WideChar);overload;override;
  public
    constructor Create(target: TNotePadString; mode: TModeJScript);
  end;

implementation

uses NotePadClasses, ParserConsts;

const
  ss1: array[0..58] of WideString = (
    'abstract',
    'boolean',
    'break',
    'byte',
    'case',
    'catch',
    'char',
    'class',
    'const',
    'continue',
    'debugger',
    'default',
    'delete',
    'do',
    'double',
    'else',
    'enum',
    'export',
    'extends',
    'false',
    'final',
    'finally',
    'float',
    'for',
    'function',
    'goto',
    'if',
    'implements',
    'import',
    'in',
    'instanceof',
    'int',
    'interface',
    'long',
    'native',
    'new',
    'null',
    'package',
    'private',
    'protected',
    'public',
    'return',
    'short',
    'static',
    'super',
    'switch',
    'synchronized',
    'this',
    'throw',
    'throws',
    'transient',
    'true',
    'try',
    'typeof',
    'var',
    'void',
    'volatile',
    'while',
    'with');

type
  THash1 = class(THashTable)
  protected
    procedure InitTable; override;
  public
    function isReserbed(const Value: WideString): boolean; override;
  end;

{ THash1 }

procedure THash1.InitTable;
var
  aa1: array of word;
  i, j: integer;
begin
  FSize := 127;{}
  FMaxLen := 1;
  SetLength(aa1, Length(ss1{}));
  for i := 0 to  Length(aa1) - 1 do begin
    aa1[i] := AnkToHash(ss1{}[i]);
    if Length(ss1{}[i]) > FMaxLen then FMaxLen := Length(ss1{}[i]);
  end;
  SetLength(FTable, FSize);
  for i := 0 to  Length(FTable) - 1 do begin
    FTable[i] := 0;
  end;
  for i := 0 to  Length(aa1) - 1 do begin
    j := aa1[i];
    while FTable[j] > 0 do begin
      inc(j);
      if j >= Length(FTable) then dec(j, Length(FTable) - 1);
    end;
    FTable[j] := i + 1;
  end;
end;

function THash1.isReserbed(const Value: WideString): boolean;
var
  i: integer;
begin
  Result := false;
  if Length(Value) > FMaxLen then exit;
  i := AnkToHash(Value);
  while FTable[i] > 0 do begin
    if ss1{}[FTable[i] - 1] = Value then begin
      Result := true;
      exit;
    end;
    inc(i);
    if i >= FSize then dec(i, FSize);
  end;
end;

var
  FHash1: THash1;

type
  TJScriptList = class(TReserveList)
  public
    function Find(const s: WideString): boolean; override;
  end;

{ TJScriptList }

function TJScriptList.Find(const s: WideString): boolean;
begin
  Result := FHash1.isReserbed(s);
end;

{ TModeJScript }

procedure TModeJScript.InitReserveList;
var
  a: array of WideString;
begin
  FReserveList := TJScriptList.Create(a, true);
  FReserve2List := TReserveList.Create(a, true);
  FReserve3List := TReserveList.Create(a, true);
end;

function TModeJScript.NewParser(target: TNotePadString): TNotePadParser;
begin
  Result := TParserJScript.Create(target, self);
end;

{ TParserJScript }

constructor TParserJScript.Create(target: TNotePadString;
  mode: TModeJScript);
begin
  inherited Create(target, mode);
end;

procedure TParserJScript.ParseToken(AHttp: boolean);
var
  C2: WideChar;
begin
  if FToken = toCommentBrock1Start then begin
    ParseToken(toCommentBrock1);
    exit;
  end;
  case FSb[FParsePos] of
    '(', '+' .. ',', '=': begin
      ParseToken(toSymbolNoDiv);
      exit;
    end;
    #$27: begin
      if FParsePos < FSb.StringLength - 1 then begin
        C2 := FSb[FParsePos + 1];
        if C2 <> #10 then begin
          inc(FParsePos);
          ParseToken(toSingleQuotation);
          exit;
        end;
      end;
      ParseToken(toSingleQuotation);
      exit;
    end;
    #$22: begin
      if FParsePos < FSb.StringLength - 1 then begin
        C2 := FSb[FParsePos + 1];
        if C2 <> #10 then begin
          inc(FParsePos);
          ParseToken(toDoubleQuotation);
          exit;
        end;
      end;
      ParseToken(toDoubleQuotation);
      exit;
    end;
    '/': begin
      if FParsePos < FSb.StringLength - 1 then begin
        C2 := FSb[FParsePos + 1];
        if C2 = '/' then begin
          inc(FParsePos);
          ParseToken(toComment);
          exit;
        end;
        if C2 = '*' then begin
          inc(FParsePos);
          ParseToken(toCommentBrock1Start);
          exit;
        end;
        if (FPrevToken = toSymbolNoDiv) and (C2 <> #10) then begin
          inc(FParsePos);
          ParseToken(toRegExp);
          exit;
        end;
      end;
      if FPrevToken = toSymbolNoDiv then begin
        ParseToken(toRegExp);
        exit;
      end;
    end;
    #$30: begin
      if FParsePos < FSb.StringLength - 1 then begin
        C2 := FSb[FParsePos + 1];
        if (C2 = #$58) or (C2 = #$78) then begin
          inc(FParsePos);
          ParseToken(toHex);
          exit;
        end;
      end;
    end;
  end;//case
  inherited ParseToken(false);
end;

procedure TParserJScript.ParseToken(const C: WideChar);
begin
  case C of
    toSingleQuotation: begin
      FToken := C;
      SingleQuotSkipProc(false);
    end;
    toSingleQuotSkip: begin
      FToken := toSingleQuotation;
      SingleQuotSkipProc(true);
    end;
    toDoubleQuotation: begin
      FToken := C;
      DoubleQuotSkipProc(false);
    end;
    toDoubleQuotSkip: begin
      FToken := toDoubleQuotation;
      DoubleQuotSkipProc(true);
    end;
    toComment: begin
      FToken := C;
      CommentProc;
    end;
    toCommentBrock1Start: begin
      FToken := C;
      CommentBrock1StartProc;
    end;
    toCommentBrock1: begin
      FToken := C;
      CommentBrock1Proc(0);
    end;
    toCommentBrock1sp: begin
      FToken := toCommentBrock1;
      CommentBrock1Proc(1);
    end;
    toHex: begin
      FToken := C;
      HexProc;
    end;
    toRegExp: begin
      FToken := C;
      RegExpSkipProc(false);
    end;
    toRegExpSkip: begin
      FToken := toRegExp;
      RegExpSkipProc(true);
    end;
    else inherited;
  end;
end;

initialization
  FHash1 := THash1.Create;
finalization
  FHash1.Free;
end.
