unit StringBuffer;

interface

type
  TStringBuffer = class
  private
    FChars : WideString;
    FLength : integer;
    function GetBufferSize : integer;
    function Get(Index: Integer): WideChar;
    procedure Put(Index: Integer; const C: WideChar); 
  public
    constructor Create(size : integer);
    destructor Destroy;override;
    procedure Append(const s: WideString);overload;
    procedure Append(p: PWideChar; ALen: integer);overload;
    procedure Insert(index: integer; const s: WideString; ALen: integer);
    procedure Remove(index, len: integer);
    procedure Clear;
    function GetString: WideString; overload;
    function GetString(start, len: integer): WideString;overLoad;
    function ToPointer(index: integer): PWideChar;
    property StringLength: integer read FLength;
    property Capacity: integer read GetBufferSize;
    property Chars[Index: Integer]: WideChar read Get write Put; default;
  end;

implementation

{ TStringBuffer }

procedure TStringBuffer.Append(const s: WideString);
var
  i: integer;
  Lens: integer;
  ps, pd: Cardinal;
begin
  Lens := Length(s);
  i := Length(FChars);
  while FLength + Lens + 1 > i do inc(i, i);
  if i > Length(FChars) then SetLength(FChars, i);
  ps := Cardinal(PWideChar(s));
  pd := Cardinal(PWideChar(FChars)) + Cardinal(FLength) * 2;
  Move(Pointer(ps)^, Pointer(pd)^, Lens * 2);
  {for i := 0 to Lens - 1 do begin
    FChars[FLength + i + 1] := s[i + 1];
  end;}
  inc(FLength, Lens);
  FChars[FLength + 1] := #0;
end;

procedure TStringBuffer.Append(p: PWideChar; ALen: integer);
var
  i: integer;
  ps, pd: Cardinal;
begin
  i := Length(FChars);
  while FLength + ALen + 1 > i do inc(i, i);
  if i > Length(FChars) then SetLength(FChars, i);
  ps := Cardinal(p);
  pd := Cardinal(PWideChar(FChars)) + Cardinal(FLength) * 2;
  Move(Pointer(ps)^, Pointer(pd)^, ALen * 2);
  inc(FLength, ALen);
  FChars[FLength + 1] := #0;
end;

procedure TStringBuffer.Clear;
begin
  FLength := 0;
  FChars[1] := #0;
end;

constructor TStringBuffer.Create(size: integer);
begin
  if size < 2 then size := 2;
  SetLength(FChars, size);
  FChars[1] := #0;
  FLength := 0;
end;

destructor TStringBuffer.Destroy;
begin
  SetLength(FChars, 0);
  inherited;
end;

function TStringBuffer.Get(Index: Integer): WideChar;
begin
  result := FChars[Index + 1];
end;

function TStringBuffer.GetBufferSize: integer;
begin
  result := Length(FChars);
end;

procedure TStringBuffer.Insert(index: integer; const s: WideString; ALen: integer);
var
  i : integer;
  ps, pd: Cardinal;
begin
  i := Length(FChars);
  while FLength + Alen + 1 > i do inc(i, i);
  if i > Length(FChars) then SetLength(FChars, i);
  ps := Cardinal(PWideChar(FChars)) + Cardinal(index) * 2;
  pd := ps + Cardinal(Alen) * 2;
  Move(Pointer(ps)^, Pointer(pd)^, (FLength - index + 1) * 2);
  pd := ps;
  ps := Cardinal(PWideChar(s));
  Move(Pointer(ps)^, Pointer(pd)^, Alen * 2);
  {for i := 1 to Lens do begin
    FChars[index + i] := s[i];
  end;}
  inc(FLength, Alen);
end;

procedure TStringBuffer.Put(Index: Integer; const C: WideChar);
begin
  FChars[Index + 1] := C;
end;

procedure TStringBuffer.Remove(index, len: integer);
var
  ps, pd: Cardinal;
begin
  pd := Cardinal(PWideChar(FChars)) + Cardinal(index) * 2;
  ps := pd + Cardinal(len) * 2;
  Move(Pointer(ps)^, Pointer(pd)^, (FLength  - len - index + 1) * 2);
  dec(FLength, len);
end;

function TStringBuffer.ToPointer(index: integer): PWideChar;
begin
  Result := PWideChar(FChars) + index;
end;

function TStringBuffer.GetString(start, len: integer): WideString;
var
//  i: integer;
  ps, pd: Cardinal;
begin
  SetLength(Result, len);
  ps := Cardinal(PWideChar(FChars)) + Cardinal(start) * 2;
  pd := Cardinal(PWideChar(Result));
  Move(Pointer(ps)^, Pointer(pd)^, (len) * 2);

  {for i := 0 to len -1 do
    Result[i + 1] := FChars[start + i + 1];}
end;

function TStringBuffer.GetString: WideString;
var
//  i: integer;
  ps, pd: Cardinal;
begin
  SetLength(Result, FLength);
  ps := Cardinal(PWideChar(FChars));
  pd := Cardinal(PWideChar(Result));
  Move(Pointer(ps)^, Pointer(pd)^, (FLength) * 2);
  {for i := 0 to FLength -1 do
    Result[i + 1] := FChars[i + 1];}
end;

end.
