unit TextFileEncoding;

interface

uses Windows, Messages, SysUtils, Classes;

type
  TFileCode=(
    fcNONE,
    fcANSI,
    fcUNICODE,
    fcBIGEND,
    fcUTF8W,
    fcUTF7,
    fcUTF8U,
    fcEUC,
    fcJIS,
    fcASCII,
    fc1251,
    fcBIG5,
    fcGB2312,
    fcEUCKR,
    fcBIN);

const
  FileCodeString: array[0..13] of WideString = (
    '',
    '[S-JIS]',
    '[Unicode]',
    '[Big Endian]',
    '[UTF8(BOM)]',
    '[UTF7]',
    '[UTF8]',
    '[EUC]',
    '[JIS]',
    '[ASCII]',
    '[L(1251)]',
    '[BIG5]',
    '[GB2312]',
    '[EUC-KR]');

function LoadFromFile(const fileName: WideString; var fileCode: TFileCode; binFix: boolean; out AError: boolean; out AFixed: boolean): WideString;
function SaveToFile(const FileName: widestring;const Text:WideString;FileCode:TFileCode):boolean;
function GetHeaderIsText(const PathName: WideString): Boolean;

implementation

uses wsFileOp, codeconv;



function CreateStream(const FileName: widestring): TFileStream;
begin
  try
    Result:=TFileStream.Create(FileName, fmOpenRead or fmShareDenyWrite);
  except
    try
       Result:=TFileStream.Create(FileName, fmOpenRead or fmShareDenyNone);
    except
       Result:=nil;
    end;
  end;
end;

function AnsiStringFromFile(const FileName: WideString; MaxSize:Integer; out AFailed: boolean): AnsiString;
var
  Stream: TStream;
  Size: Integer;
begin
  AFailed := true;
  Stream := CreateStream(FileName);
  try
    if Stream <> nil then begin
      Size := Stream.Size - Stream.Position;
      if MaxSize < Size then Size := MaxSize;
      SetString(Result, nil, Size);
      Stream.Read(Pointer(Result)^, Size);
      AFailed := false;
    end;
  finally
    Stream.Free;
  end;
end;

function GetHeaderIsText(const PathName: WideString): Boolean;
var
  l, i, ii: integer;
  s: AnsiString;
  b: boolean;

begin
  Result := false;
  s := AnsiStringFromFile(PathName, 32, b);
  if b then Exit;
  l := Length(s);
  if l > 0 then begin
    ii := 0;
    for i := 1 to l do if byte(s[i]) <= 7 then inc(ii);
    if (ii = 0) or
       ((l >= 2) and (integer(byte(s[1])) + integer(byte(s[2])) >= 509))
    then Result := true;
  end else Result := true;
end;

function LoadFromFile(const fileName: WideString; var fileCode: TFileCode; binFix: boolean; out AError: boolean; out AFixed: boolean): WideString;
var
  Value: AnsiString;
  c: AnsiChar;
  i: integer;
  p, p2: PAnsiChar;
  b: boolean;
  IC: integer;
//label
//  BinCheckW;
  procedure BinCheck;
  var
    ii: integer;
  begin
    if binFix then begin
      for ii := 1 to Length(Result) do if Result[ii] = #0 then begin
        Result[ii] := #$20;
        AFixed := true;
      end;
    end else begin
      for ii := 1 to Length(Result) do if Result[ii] = #0 then begin
        Result := '';
        AFixed := true;
        exit;
      end;
    end;
  end;
begin
  Result := '';
  AFixed := false;
  Value := AnsiStringFromFile(fileName, MaxInt, AError);
  if AError then Exit;
  case fileCode of
    fcNONE: begin
      FileCode := fcANSI;//ANSI
      b := false;
      if Length(Value) > 1 then begin
        if (byte(Value[1]) > 253) and (byte(Value[2]) > 253) then b := true;
        if ((byte(Value[1]) = 0) and (byte(Value[2]) <> 0)) or ((byte(Value[1]) <> 0) and (byte(Value[2]) = 0)) then b := true;
        if b then begin
          fileCode := fcUNICODE;//unicode
          P := Pointer(Value);
          if ((p^ = #254) and ((p + 1)^ = #255)) or (p^ = #0) then begin
            i := Length(Value);
            while i > 0 do begin
              c := p^;
              p^ := (p + 1)^;
              (p + 1)^ := c;
              inc(p,2);
              dec(i,2);
            end;//while
            fileCode := fcBIGEND;//unicode big
          end;//if
          P := Pointer(Value);
          if (p^ = #255) and ((p + 1)^ = #254) then begin
            SetLength(Result, Length(Value) div 2 - 1);
            p := Pointer(Value);
            p2 := Pointer(Result);
            inc(p, 2);
            for i := 2 to Length(Value) - 1 do begin
              p2^ := p^;
              inc(p);
              inc(p2);
            end;
          end else begin
            SetLength(Result, Length(Value) div 2);
            p := Pointer(Value);
            p2 := Pointer(Result);
            for i:=0 to Length(Value)-1 do begin
              p2^ := p^;
              inc(p);
              inc(p2);
            end;
          end;
          BinCheck;
          exit;
        end;//if b
        if ((Value[1] = #$EF) and (Value[2] = #$BB)) and
           (Length(Value) > 2) and (Value[3] = #$BF) then begin
          SetLength(Result, Length(Value));
          SetLength(Result, MultiByteToWideChar(CP_UTF8,
                                                0,
                                                PAnsiChar(Value),
                                                Length(Value),
                                                PWideChar(Result),
                                                Length(Result))
                   );
          if Length(Result) > 0 then Delete(Result, 1, 1);//UTF8NOBOM
          fileCode := fcUTF8W;//UTF8BOM
          Bincheck;
          exit;
        end;
      end;  //if

      if binFix then begin
        for i := 1 to Length(Value) do if Value[i] = #0 then begin
          Value[i] := #$20;
          AFixed := true;
        end;
      end else begin
        for i:=1 to Length(Value) do if Value[i] = #0 then begin
          Result := '';
          exit;
        end;
      end;
      ic := EncodingCheck(Value, Length(Value));
      case IC of
        ENC_UTF7:begin
          SetLength(Result, Length(Value));
          SetLength(Result, MultiByteToWideChar(CP_UTF7,
                                                0,
                                                PAnsiChar(Value),
                                                Length(Value),
                                                PWideChar(Result),
                                                Length(Result))
                   );
          fileCode := fcUTF7;
        end;
        ENC_UTF8:begin
          SetLength(Result, Length(Value));
          SetLength(Result, MultiByteToWideChar(CP_UTF8,
                                                0,
                                                PAnsiChar(Value),
                                                Length(Value),
                                                PWideChar(Result),
                                                Length(Result))
                   );
          fileCode := fcUTF8U;
        end;
        ENC_EUC:begin
          Result := Euc2Wide(Value);
          fileCode := fcEUC;
        end;
        ENC_JIS: begin
          Result := Jis2Wide(Value);
          fileCode := fcJIS;
        end;
        else begin
          SetLength(Result, Length(Value));
          SetLength(Result, MultiByteToWideChar(932,
                                                0,
                                                PAnsiChar(Value),
                                                Length(Value),
                                                PWideChar(Result),
                                                Length(Result))
                   );
          //Result := Value;
        end;
      end;
      if Length(Result) = 0 then fileCode := fcAnsi;
      exit;//AnsiŃ`FbNς
    end;
    fcANSI: begin
      SetLength(Result, Length(Value));
      SetLength(Result, MultiByteToWideChar(932,
                                            0,
                                            PAnsiChar(Value),
                                            Length(Value),
                                            PWideChar(Result),
                                            Length(Result))
               );
      //Result := Value;
    end;
    fcUNICODE: begin
      P := Pointer(Value);
      if (p^ = #255) and ((p + 1)^ = #254) then begin
        SetLength(Result, Length(Value) div 2 - 1);
        p := Pointer(Value);
        p2 := Pointer(Result);
        inc(p, 2);
        for i := 2 to Length(Value) - 1 do begin
          p2^ := p^;
          inc(p);
          inc(p2);
        end;
      end else begin
        SetLength(Result, Length(Value) div 2);
        p := Pointer(Value);
        p2 := Pointer(Result);
        for i:=0 to Length(Value) - 1 do begin
          p2^ := p^;
          inc(p);
          inc(p2);
        end;
      end;
    end;
    fcBIGEND: begin
      P := Pointer(Value);
      i := Length(Value);
      while i > 0 do begin
        c := p^;
        p^ := (p + 1)^;
        (p + 1)^ := c;
        inc(p, 2);
        dec(i, 2);
      end;//while
      P := Pointer(Value);
      if (p^ = #255) and ((p + 1)^ = #254) then begin
        SetLength(Result, Length(Value) div 2 - 1);
        p := Pointer(Value);
        p2 := Pointer(Result);
        inc(p,2);
        for i := 2 to Length(Value) - 1 do begin
          p2^ := p^;
          inc(p);
          inc(p2);
        end;
      end else begin
        SetLength(Result, Length(Value) div 2);
        p := Pointer(Value);
        p2 := Pointer(Result);
        for i := 0 to Length(Value) - 1 do begin
          p2^ := p^;
          inc(p);
          inc(p2);
        end;
      end;
    end;
    fcUTF8W, fcUTF8U: begin
      //Result:=Utf8Decode(value);
      SetLength(Result, Length(Value));
      SetLength(Result, MultiByteToWideChar(CP_UTF8,
                                            0,
                                            PAnsiChar(Value),
                                            Length(Value),
                                            PWideChar(Result),
                                            Length(Result))
               );
      if (Length(Value) > 2) and ((Value[1] = #$EF) and (Value[2] = #$BB)) then
        Delete(Result, 1, 1);
    end;
    fcUTF7: begin
      SetLength(Result, Length(Value));
      SetLength(Result, MultiByteToWideChar(CP_UTF7,
                                            0,
                                            PAnsiChar(Value),
                                            Length(Value),
                                            PWideChar(Result),
                                            Length(Result))
               );
    end;
    fcEUC: begin
        //Result:=euc2sjis(value);
      Result:=Euc2Wide(value);
    end;
    fcJIS: begin
        //Result:=jis2sjis(value);
      Result:=Jis2Wide(value);
    end;
    fcASCII: begin
      p := Pointer(Value);
      i := 0;
      while i < Length(Value) do begin
        if (p + i)^ = #0 then (p + i)^ := #$20;
        inc(i);
      end;
      SetLength(Result, Length(Value));
      SetLength(Result, MultiByteToWideChar(1252,
                                            MB_PRECOMPOSED,
                                            PAnsiChar(Value),
                                            Length(Value),
                                            PWideChar(Result),
                                            Length(Result))
               );
    end;
    fc1251: begin
      SetLength(Result, Length(Value));
      SetLength(Result, MultiByteToWideChar(1251,
                                            MB_PRECOMPOSED,
                                            PAnsiChar(Value),
                                            Length(Value),
                                            PWideChar(Result),
                                            Length(Result))
      );
    end;
    fcBIG5: begin
      SetLength(Result, Length(Value));
      SetLength(Result, MultiByteToWideChar(950,
                                            MB_PRECOMPOSED,
                                            PAnsiChar(Value),
                                            Length(Value),
                                            PWideChar(Result),
                                            Length(Result))
      );
    end;
    fcGB2312: begin
      SetLength(Result, Length(Value));
      SetLength(Result, MultiByteToWideChar(936,
                                            MB_PRECOMPOSED,
                                            PAnsiChar(Value),
                                            Length(Value),
                                            PWideChar(Result),
                                            Length(Result))
               );
    end;
    fcEUCKR: begin
      SetLength(Result, Length(Value));
      SetLength(Result, MultiByteToWideChar(949,
                                            MB_PRECOMPOSED,
                                            PAnsiChar(Value),
                                            Length(Value),
                                            PWideChar(Result),
                                            Length(Result))
               );
    end;
    fcBIN: ;
  end;
  BinCheck;
end;

function SaveToFile(const FileName: widestring;const Text:WideString;FileCode:TFileCode):boolean;
var
  Stream: TStream;
  s: AnsiString;
  ss: WideString;
  c: AnsiChar;
  p: PAnsiChar;
  i:integer;
  usedefchar:bool;
begin
  try
  case FileCode of
  fcUNICODE:begin
    ss:=#$feff+Text;
    p:=pointer(ss);
    i:=length(SS)*2;
    SetString( s, p, i );
  end;//2
  fcBIGEND:begin
    ss:=#$feff+Text;
    p:=pointer(ss);
    i:=length(SS)*2;
    while i>0 do begin
      c:=p^;p^:=(p+1)^;(p+1)^:=c;
      inc(p,2);dec(i,2);
    end;//while
    p:=pointer(ss);
    i:=length(SS)*2;
    SetString( s, p, i );
  end;//3
  //UTF8Encode̎dltiburonύXꂽ炵B
  fcUTF8W:begin
    SetLength(s,WideCharToMultiByte(CP_UTF8,0,pwchar(Text),Length(Text),nil,0,nil,nil));
    WideCharToMultiByte(CP_UTF8,0,pwchar(Text),Length(Text),pansichar(s),length(s),nil,nil);
    s := AnsiChar(#$EF)+AnsiChar(#$BB)+AnsiChar(#$BF) + s;
    (* us:=#$EF#$BB+#$BF+UTF8Encode(Text);
    p:=pointer(us);
    i:=length(us);
    SetString( s, p, i ); *)
  end;//4
  fcUTF8U:begin
    SetLength(s,WideCharToMultiByte(CP_UTF8,0,pwchar(Text),Length(Text),nil,0,nil,nil));
    WideCharToMultiByte(CP_UTF8,0,pwchar(Text),Length(Text),pansichar(s),length(s),nil,nil);
    (* us:={#$EF#$BB+#$BF+}UTF8Encode(Text);
    p:=pointer(us);
    i:=length(us);
    SetString( s, p, i ); *)
  end;//5
  fcEUC:begin
    //s:=sjis2euc(Text);
    s:=Wide2Euc(Text, false, false);
  end;
  fcJIS:begin
    //s:=sjis2jis83(Text);
    s:=Wide2Jis(Text);
  end;
  fcUTF7:begin
    SetLength(s,WideCharToMultiByte(CP_UTF7,0,pwchar(Text),Length(Text),nil,0,nil,nil));
    WideCharToMultiByte(CP_UTF7,0,pwchar(Text),Length(Text),pansichar(s),length(s),nil,nil);
  end;
  fcASCII:begin
    usedefchar:=false;
    SetLength(s,WideCharToMultiByte(1252,0{WC_COMPOSITECHECK or WC_SEPCHARS or WC_DEFAULTCHAR},pwchar(Text),Length(Text),nil,0,nil,@usedefchar));
    WideCharToMultiByte(1252,0{WC_COMPOSITECHECK or WC_SEPCHARS or WC_DEFAULTCHAR},pwchar(Text),Length(Text),PAnsiChar(s),length(s),nil,@usedefchar);
  end;
  fc1251:begin
    usedefchar:=false;
    SetLength(s,WideCharToMultiByte(1251,0{WC_COMPOSITECHECK or WC_SEPCHARS or WC_DEFAULTCHAR},pwchar(Text),Length(Text),nil,0,nil,@usedefchar));
    WideCharToMultiByte(1251,0{WC_COMPOSITECHECK or WC_SEPCHARS or WC_DEFAULTCHAR},pwchar(Text),Length(Text),PAnsiChar(s),length(s),nil,@usedefchar);
  end;
  fcBIG5:begin
    usedefchar:=false;
    SetLength(s,WideCharToMultiByte(950,0{WC_COMPOSITECHECK or WC_SEPCHARS or WC_DEFAULTCHAR},pwchar(Text),Length(Text),nil,0,nil,@usedefchar));
    WideCharToMultiByte(950,0{WC_COMPOSITECHECK or WC_SEPCHARS or WC_DEFAULTCHAR},pwchar(Text),Length(Text),PAnsiChar(s),length(s),nil,@usedefchar);
  end;
  fcGB2312:begin
    usedefchar:=false;
    SetLength(s,WideCharToMultiByte(936,0{WC_COMPOSITECHECK or WC_SEPCHARS or WC_DEFAULTCHAR},pwchar(Text),Length(Text),nil,0,nil,@usedefchar));
    WideCharToMultiByte(936,0{WC_COMPOSITECHECK or WC_SEPCHARS or WC_DEFAULTCHAR},pwchar(Text),Length(Text),PAnsiChar(s),length(s),nil,@usedefchar);
  end;
  fcEUCKR:begin
    usedefchar:=false;
    SetLength(s,WideCharToMultiByte(949,0{WC_COMPOSITECHECK or WC_SEPCHARS or WC_DEFAULTCHAR},pwchar(Text),Length(Text),nil,0,nil,@usedefchar));
    WideCharToMultiByte(949,0{WC_COMPOSITECHECK or WC_SEPCHARS or WC_DEFAULTCHAR},pwchar(Text),Length(Text),PAnsiChar(s),length(s),nil,@usedefchar);
  end;
  fcBIN: begin
  //base64ô߂̃ubN
    ss := Text;
    p := pointer(ss);
    i := length(SS) * 2;
    while i > 0 do begin
      c:=p^;p^:=(p+1)^;(p+1)^:=c;
      inc(p,2);dec(i,2);
    end;//while
    p := pointer(ss);
    i := length(SS) * 2;
    SetString( s, p, i );
  end;
  else begin
    //IvV̓[ɂƃVXeIɕϊłH
    usedefchar:=false;
    SetLength(s, WideCharToMultiByte(932, 0, PWChar(Text), Length(Text), nil, 0, nil, @usedefchar));
    WideCharToMultiByte(932, 0, PWChar(Text), Length(Text), PAnsiChar(s), length(s), nil, @usedefchar);
    //MessageBoxA(0,PAnsiChar(s),nil,0);
    //s:=Text;
  end;
  end;//case
    Result := true;
  except
    Result := false;
  end;
  if not Result then exit;
  p:=pointer(s);
  i:=length(s);
  try
    Stream := TFileStream.Create(FileName, fmCreate);
    try
      try
        Stream.WriteBuffer(p^, i);
        Result := true;
      except
        Result := false;
      end;
    finally
      Stream.Free;
//      Stream := nil;
    end;
  except
//    if Assigned(Stream) then Stream.Free;
    result := false;
  end;
end;
end.
